{{
HD61830 Driver

LCD Module Pin-out (According to LCD module Datasheet)
------------------------------------------------------
1       GND                     Ground (0V)
2       VDD                     Power Supply for logic circuit (5V)
3       V0                      Operating voltage for LCD driving (Variable)
4       D/I    (aka RS)         H:Data input L:Instruction code input
5       R/W                     H:Data read (LCM to MPU) L:Data write (MPU to LCM)
6       E                       Enable
7       DB0                     Data Bus
8       DB1                     Data Bus
9       DB2                     Data Bus
10      DB3                     Data Bus
11      DB4                     Data Bus
12      DB5                     Data Bus
13      DB6                     Data Bus
14      DB7                     Data Bus
15      CS                      Chip Select
16      RST                     Reset (Active "LOW")
17      VOUT                    Output Voltage for LCD Driving

Note:
  The LCD Module that was used in writing this driver was a Solomon LM6231SGR (good luck finding info on it)
  unfortunately this module uses a split VRAM causing a need to write every character twice in text mode, will
  explain more in putchar function.
  



LCD Registers/Commands
----------------------------
LCD Mode                $00     Sets Text or Graphics mode, cursor setup and enables display
Character Pitch         $01     Set internal font size 5x7 ($75) or 5x11 ($B5), set to $07 when in graphics mode
Chars per line          $02     Sets number of text chars per line in txt mode or it's sets the number of pixels per line in graphics mode
Duty Cycle              $03     Sets duty cycle for lcd module
Cursor Position         $04     Set cursor position
Display Start Addr Low  $08     Set's low byte of starting memory address that will show on display
Display Start Addr High $09     Set's high byte of memory address that will show on display
Cursor Address Low      $0A     Set's low byte of cursor location
Cursor Address High     $0B     Set's high byte of cursor location
Write Data to VRAM      $0C     Write data to VRAM
Read Data from VRAM     $0D     Read data from VRAM (not used in this driver)
Clear bit in VRAM       $0E     Set VRAM memory location to low
Set Bit in VRAM         $0F     Set VRAM memory location to high
 


}}

con
''Display definitions
  horiz_text    = 40               ' Sets number of possible characters per line for text mode
  vert_text     = 8                ' Sets number of possible text lines
  horiz_pixels  = 240              ' Sets number of pixels per line for graphics mode
  vert_pixels   = 64               ' Sets number of lines of pixles
  duty_cycle    = 64               ' Sets duty cycle of the lcd

''Pin definitions
  databus_pins  = 0                ' Sets first pin of an eight pin group used for data lines
  enable_pin    = 8                ' Sets pin to use for enable line
  rs_pin        = 9                ' Sets pin to use for rs line
  rw_pin        = 10               ' Sets pin to use for rw line
  cs_pin        = 11               ' Sets pin to use for cs line
  reset_pin     = 12               ' Sets pin to use for reset line
  
var
''Place holder
  byte Mode

obj
  Delay : "Timing"
  debugterm : "FullDuplexSerial"

pub InitDisplay
'' Sets the pins needed to output, sets there initial stats, and resets the display.

  debugterm.start(31, 30, 0, 57600)

  dira[databus_pins..(databus_pins+7)]~~           ' Sets the data bus pins to output
  dira[enable_pin]~~                               ' Sets the enable pin to output
  dira[rs_pin]~~                                   ' Sets the rs pin to output
  dira[rw_pin]~~                                   ' Sets the rw pin to output
  dira[cs_pin]~~                                   ' Sets the cs pin to output
  dira[reset_pin]~~                                ' Sets the reset pin to output
  
  outa[enable_pin]~                                ' Sets the enable pin to low
  outa[reset_pin]~                                 ' Sets the enable pin to low, reset the lcd
  outa[rs_pin]~                                    ' Sets the enable pin to low
  outa[rw_pin]~                                    ' Sets the enable pin to low
  outa[cs_pin]~~                                   ' Sets the cs pin to high
  Delay.pause1ms( 10 )                             ' Add a delay of 10ms leaving the reset pin low to make sure we get a reset
  outa[reset_pin]~~                                ' Sets the reset pin to high, to bring lcd out of reset
  Delay.pause1ms( 10 )                             ' Add a delay of 10ms to make sure everything comes out of reset

  

pri putdata( reg, data_byte )
'' Sends two bytes of information to the lcd first byte "reg" is sent with rs high, second byte "data" is sent with rs low.

  busy_check                                            ' Check to see if LCD is busy, lets not be rude
  outa[cs_pin]~                                         ' Set chip select low, let the lcd know we are talking to him

  outa[rs_pin]~~                                        ' Set rs line to high to let the lcd know this byte is the register address
  outa[rw_pin]~                                         ' Set rw line to low, so the chip knows to listen and not speak
  outa[(databus_pins+7)..databus_pins] := (reg & $FF)   ' Set our data lines equal to the low byte of our reg variable
  lcde_pulse                                            ' Set our enable pin high, wait, and then low again.  This lets the lcd know
                                                        ' that it's time for him to read the data lines
    
  outa[rs_pin]~                                                  ' Set rs line to low to let the lcd know that this byte is the data to put at the above address
  outa[(databus_pins+7)..databus_pins] := (data_byte & $FF)      ' Set our data lines equal to the low byte of our data_byte variable
  lcde_pulse                                                     ' Set our enable pin high, wait, and then low again.  This lets the lcd know
                                                                 ' that it's time for him to read the data lines

  outa[cs_pin]~~                                         ' Set chip select high, let the lcd know we done talking to him


pri lcde_pulse
'' Sets the enable line high, waits, and then sets the enable line low
  outa[enable_pin]~~                 ' Set enable line high
  waitcnt( clkfreq/200_000 + cnt)    ' wait for clkfreq/200_000, which is 400 at 80MHz
  outa[enable_pin]~                  ' Set enable line low


pri busy_check | checktimes
'' Check to see if lcd is busy repeatedly, if lcd hasn't reported ready after 500 tries just
'' assume lcd is ready.

  checktimes := 500                   ' Set number of times to check for ready state until giving up, and assuming ready

  dira[databus_pins+7]~               ' Set highest data line to input so we can listen for a ready
  outa[rs_pin]~~                      ' Set rs line high
  outa[rw_pin]~~                      ' Set rw line high to let the lcd know we are listening
  outa[cs_pin]~                       ' Set cs line low so that the lcd knows we are expecting something from him

  repeat while checktimes > 0         ' Okay we want to repeat till checktimes is 0
    
    outa[enable_pin]~~                ' Set enable line to high so that the lcd knows it's time to speak
    waitcnt( clkfreq/200_000 + cnt)   ' Give him time to set the line before we read it
    checktimes := checktimes * ina[databus_pins+7]    ' Set checktimes equal to checktimes multiplied by input from highest data line
                                                      ' this will zero checktimes if lcd is ready
    checktimes--                      ' subtract one from checktimes 
    outa[enable_pin]~                 ' Let lcd know that we are done with information he put on the line

  outa[cs_pin]~~                      ' Set cs line back to high so that the lcd know we took a break from talking to him
  outa[rw_pin]~                       ' Set rw line back to low so that the lcd knows we don't plan on listening anymore for now
  dira[databus_pins+7]~~              ' Set highest data line back to output so we can write full bytes again




pub SetMode( lcd_mode )
'' Set lcd mode to either text or graphics use t for text and anything else for graphics

  if lcd_mode == "t"                  ' Look to see if we want text mode 
    putdata($00, $30)                 ' Set the mode register to text mode
    putdata($01, $75)                 ' Set font to 8x6 (other option is 12x6, for extended chars)
    putdata($02, horiz_text-1)        ' Set the number of chars per line
    putdata($03, duty_cycle-1)        ' Set our duty_cycle
    putdata($04, $07)                 ' Set cursor position            
    Mode := 1                         ' Set our Mode variable to one so that clear screen knows we are in text mode
  else                                ' If not text mode it must be graphics
    putdata($00, $32)                 ' Set the mode register to graphics mode
    putdata($01, $77)                 ' Set 8 pixels per byte or something like that, bit fuzzy on details here
    putdata($02, horiz_pixels-1)      ' Set number of pixels per line
    putdata($03, duty_cycle-1)        ' Set our duty_cycle
    putdata($04, $07)                 ' Set cursor position always $07 for graphics
    Mode := 2                         ' Set our Mode variable to two so that clear screen knows graphics all the way baby

  putdata($08, $0)                    ' Set start memory address low byte to 0
  putdata($09, $0)                    ' Set start memory address high byte to 0
  putdata($0A, $0)                    ' Set current cursor location low byte to 0
  putdata($0B, $0)                    ' Set current cursor location high byte to 0

pub clear_screen 
'' Sets all the visible memory locations to 0

  putdata($0A, $0 )                   ' Set low byte of memory address to 0
  putdata($0B, $0 )                   ' Set high byte of memory address to 0

  if Mode == 1                        ' Check if it's text mode or not
    repeat horiz_text*vert_text       ' Repeats for every text location on screen
      putdata($0C, $0 )               ' Set current location to a blank (let it auto increment address location)
    putdata($0A, $0 )                 ' Set it to the upper memory block that handles every other line on the screen
    putdata($0B, $10 )                ' Yeah yeah if you don't understand A and B go together by now then you should have read the other stuff first
    repeat horiz_text*vert_text       ' Repeats for every text location on screen
      putdata($0C, $0 )               ' Repeats for every text location on screen
  else                                ' If its not text then it should be graphics
    repeat (horiz_pixels*vert_pixels) ' Repeats for every 8 pixels on display
      putdata($0C, $0)                ' Wooohooo write 8 pixels at a time
                                      ' ***Hmmmm need to try and speed this up***


pub putchar( xpos, ypos, char )
'' Put character at specific x and y location

  xpos += (ypos * horiz_text)         ' Calculate memory address from x and y 
  putdata($0A, (xpos & $FF) )         ' Set low byte of memory address
  putdata($0B, (xpos>>8))             ' Set high byte of memory address
  putdata($0C, char )                 ' Set char in memory

  '' Due to the split VRAM nature of my lcd I am required to write each char to two memory locations
  '' first to the normal memory location and the to the memory location plus $1000
  '' As a side effect of the dual memory locations I can no longer use the auto increment function of
  '' the lcd module :(

  ''If your display does not have split ram, you will most likely want to comment these lines
  putdata($0A, (xpos & $FF) )         ' Set low byte of memory address
  putdata($0B, $10 + (xpos>>8) )      ' Set high byte of memory address
  putdata($0C, char )                 ' Set char in memory
  ''-----------------------------------------------------------------------------------------

pub set_pixel( xpos, ypos, state )  | address
'' Sets the pixel to on or off depending on state ( 1 or 0 only )

  address := xpos>>3 + ypos*112       ' Calculate memory address from x and y
  '' Note: My crazy display has a weird memory layout so this may need to be changed for others 

  
  putdata( $0A, ((address) & $FF) )   ' Set address
  putdata( $0B, ((address) >> 8 ) )   ' ditto
  putdata( $0E+state, (xpos & 7))     ' Explanation of what I did here:
                                      ' $0E is register for clear and $0F is register for set so I just add state to $0E to get what I want
                                      ' Even though we already set our address we have to set the exact location within the byte of memory that it is at
                                      ' that's where the xpos&7 comes in it gives us the location within that byte that we need

pub gputchar( x, y, char ) | char_offset, xc, yc
'' Puts a char from the internal char rom on lcd

  char_offset := $8000 + ((char & $FE) * $40 )                              ' Calculate char offset in ROM

  repeat yc from char_offset to char_offset + $7F step 4                    ' Repeat for every line of char
    repeat xc from (char & 1) to 31 step 2                                  ' Repeat for every bit in each line 
      set_pixel( x+xc/2, y+((yc-char_offset)/4), ((long[yc] >> xc) & 1))    ' Put dot on display in the correct location
    
pub puts( x, y, tstring )

  if Mode == 1                              ' If lcd in text mode
    repeat strsize( tstring )               ' repeat for entire string
      putchar( x++, y, byte[tstring++] )    ' put on lcd using text mode putchar, incr x
  else                                      ' else
    repeat strsize( tstring )               ' repeat for entire string
      gputchar( x, y, byte[tstring++]  )    ' put on lcd using graphic mode putchar (aka gputchar)
      x += 16                               ' inc x by 16 to allow for next char

    